==> muffin_kafka-0.0.12-py3-none-any.whl: muffin_kafka/plugin.py <==
from __future__ import annotations

from asyncio import Task, gather
from collections import defaultdict
from collections.abc import Awaitable
from typing import Any, Callable, ClassVar, Coroutine, Dict, List, Optional, Tuple, TypedDict, cast

from aiokafka import AIOKafkaConsumer, AIOKafkaProducer, helpers
from aiokafka.client import create_task
from asgi_tools._compat import json_dumps
from muffin.plugins import BasePlugin, PluginError

TCallable = Callable[..., Awaitable[Any]]
TErrCallable = Callable[[BaseException], Awaitable[Any]]


class Options(TypedDict):
    auto_offset_reset: str
    bootstrap_servers: str
    client_id: str
    enable_auto_commit: bool
    group_id: Optional[str]
    max_poll_records: Optional[int]
    request_timeout_ms: int
    retry_backoff_ms: int
    sasl_mechanism: str
    sasl_plain_password: Optional[str]
    sasl_plain_username: Optional[str]
    security_protocol: str
    ssl_cafile: Optional[str]
    produce: bool
    auto_connect: bool


class KafkaPlugin(BasePlugin):
    name = "kafka"
    defaults: ClassVar[Options] = {
        "auto_offset_reset": "earliest",
        "bootstrap_servers": "localhost:9092",
        "client_id": "muffin",
        "enable_auto_commit": False,
        "group_id": None,
        "max_poll_records": None,
        "request_timeout_ms": 30000,
        "retry_backoff_ms": 1000,
        "sasl_mechanism": "PLAIN",
        "sasl_plain_password": None,
        "sasl_plain_username": None,
        "security_protocol": "SASL_PLAINTEXT",
        "ssl_cafile": None,
        "produce": False,
        "auto_connect": True,
    }

    producer: AIOKafkaProducer
    consumers: ClassVar[Dict[str, AIOKafkaConsumer]] = {}
    handlers: ClassVar[List[Tuple[Tuple[str, ...], Callable[..., Coroutine]]]] = []

    def __init__(self, *args, **kwargs):
        self.map = defaultdict(list)
        self.tasks: List[Task] = []
        self.error_handler = None
        super().__init__(*args, **kwargs)

    async def startup(self):
        logger = self.app.logger
        logger.info("Kafka: Starting plugin")

        if self.cfg.auto_connect:
            await self.connect()

    async def shutdown(self):
        self.app.logger.info("Stopping Kafka plugin")
        for task in self.tasks:
            task.cancel()

        if self.cfg.auto_connect:
            await gather(*[consumer.stop() for consumer in self.consumers.values()])

    async def send(self, topic: str, value: Any):
        """Send a message to Kafka."""
        if not self.cfg.produce:
            raise PluginError("Kafka: Producer is not enabled")

        if not isinstance(value, str):
            value = json_dumps(value)

        fut = await self.producer.send(topic, value)
        return fut

    def handle_topics(self, *topics: str) -> Callable[[TCallable], TCallable]:
        """Register a handler for Kafka messages."""

        def wrapper(fn):
            self.handlers.append((topics, fn))
            return fn

        return wrapper

    def handle_error(self, fn: TErrCallable) -> TErrCallable:
        """Register a handler for Kafka errors."""

        self.error_handler = fn
        return fn

    async def connect(self, *only: str):
        cfg = self.cfg
        params = {
            "auto_offset_reset": cfg.auto_offset_reset,
            "bootstrap_servers": cfg.bootstrap_servers,
            "client_id": cfg.client_id,
            "enable_auto_commit": cfg.enable_auto_commit,
            "group_id": cfg.group_id,
            "max_poll_records": cfg.max_poll_records,
            "request_timeout_ms": cfg.request_timeout_ms,
            "retry_backoff_ms": cfg.retry_backoff_ms,
            "sasl_mechanism": cfg.sasl_mechanism,
            "sasl_plain_password": cfg.sasl_plain_password,
            "sasl_plain_username": cfg.sasl_plain_username,
            "security_protocol": cfg.security_protocol,
        }
        if cfg.ssl_cafile:
            params["ssl_context"] = helpers.create_ssl_context(cafile=cfg.ssl_cafile)

        logger = self.app.logger
        logger.info("Kafka: Connecting to %s", self.cfg.bootstrap_servers)
        logger.info("Kafka: Params %r", params)

        for topics, fn in self.handlers:
            filtered = [t for t in topics if t in only] if only else topics
            for topic in filtered:
                if topic not in self.consumers:
                    logger.info("Kafka: Listen to %s", topic)
                    consumer = self.consumers[topic] = AIOKafkaConsumer(topic, **params)
                    await consumer.start()

                self.map[topic].append(fn)

        self.tasks = [create_task(self.__process__(consumer)) for consumer in self.consumers]

    async def __process__(self, consumer):
        logger = self.app.logger
        logger.info("Start listening Kafka messages")
        async for msg in consumer:
            logger.info("Kafka msg: %s-%s-%s", msg.topic, msg.partition, msg.offset)
            for fn in self.map[msg.topic]:
                try:
                    await fn(msg)
                except Exception as exc:  # noqa: PERF203
                    logger.exception("Kafka: Error while processing message: %r", msg)
                    error_handler = cast(TCallable, self.error_handler)
                    if error_handler:
                        await error_handler(exc)

==> muffin_kafka-0.0.13-py3-none-any.whl: muffin_kafka/plugin.py <==
from __future__ import annotations

from asyncio import Task, gather
from collections import defaultdict
from collections.abc import Awaitable
from typing import Any, Callable, ClassVar, Coroutine, Dict, List, Optional, Tuple, TypedDict, cast

from aiokafka import AIOKafkaConsumer, AIOKafkaProducer, helpers
from aiokafka.client import create_task
from asgi_tools._compat import json_dumps
from muffin.plugins import BasePlugin, PluginError

TCallable = Callable[..., Awaitable[Any]]
TErrCallable = Callable[[BaseException], Awaitable[Any]]


class Options(TypedDict):
    auto_offset_reset: str
    bootstrap_servers: str
    client_id: str
    enable_auto_commit: bool
    group_id: Optional[str]
    max_poll_records: Optional[int]
    request_timeout_ms: int
    retry_backoff_ms: int
    sasl_mechanism: str
    sasl_plain_password: Optional[str]
    sasl_plain_username: Optional[str]
    security_protocol: str
    ssl_cafile: Optional[str]
    produce: bool
    auto_connect: bool


class KafkaPlugin(BasePlugin):
    name = "kafka"
    defaults: ClassVar[Options] = {
        "auto_offset_reset": "earliest",
        "bootstrap_servers": "localhost:9092",
        "client_id": "muffin",
        "enable_auto_commit": False,
        "group_id": None,
        "max_poll_records": None,
        "request_timeout_ms": 30000,
        "retry_backoff_ms": 1000,
        "sasl_mechanism": "PLAIN",
        "sasl_plain_password": None,
        "sasl_plain_username": None,
        "security_protocol": "SASL_PLAINTEXT",
        "ssl_cafile": None,
        "produce": False,
        "auto_connect": True,
    }

    producer: AIOKafkaProducer
    consumers: ClassVar[Dict[str, AIOKafkaConsumer]] = {}
    handlers: ClassVar[List[Tuple[Tuple[str, ...], Callable[..., Coroutine]]]] = []

    def __init__(self, *args, **kwargs):
        self.map = defaultdict(list)
        self.tasks: List[Task] = []
        self.error_handler = None
        super().__init__(*args, **kwargs)

    async def startup(self):
        logger = self.app.logger
        logger.info("Kafka: Starting plugin")

        if self.cfg.auto_connect:
            await self.connect()

    async def shutdown(self):
        self.app.logger.info("Stopping Kafka plugin")
        for task in self.tasks:
            task.cancel()

        if self.cfg.auto_connect:
            await gather(*[consumer.stop() for consumer in self.consumers.values()])

    async def send(self, topic: str, value: Any):
        """Send a message to Kafka."""
        if not self.cfg.produce:
            raise PluginError("Kafka: Producer is not enabled")

        if not isinstance(value, str):
            value = json_dumps(value)

        return await self.producer.send(topic, value)

    def handle_topics(self, *topics: str) -> Callable[[TCallable], TCallable]:
        """Register a handler for Kafka messages."""

        def wrapper(fn):
            self.handlers.append((topics, fn))
            return fn

        return wrapper

    def handle_error(self, fn: TErrCallable) -> TErrCallable:
        """Register a handler for Kafka errors."""

        self.error_handler = fn
        return fn

    async def connect(self, *only: str):
        cfg = self.cfg
        params = {
            "auto_offset_reset": cfg.auto_offset_reset,
            "bootstrap_servers": cfg.bootstrap_servers,
            "client_id": cfg.client_id,
            "enable_auto_commit": cfg.enable_auto_commit,
            "group_id": cfg.group_id,
            "max_poll_records": cfg.max_poll_records,
            "request_timeout_ms": cfg.request_timeout_ms,
            "retry_backoff_ms": cfg.retry_backoff_ms,
            "sasl_mechanism": cfg.sasl_mechanism,
            "sasl_plain_password": cfg.sasl_plain_password,
            "sasl_plain_username": cfg.sasl_plain_username,
            "security_protocol": cfg.security_protocol,
        }
        if cfg.ssl_cafile:
            params["ssl_context"] = helpers.create_ssl_context(cafile=cfg.ssl_cafile)

        logger = self.app.logger
        logger.info("Kafka: Connecting to %s", self.cfg.bootstrap_servers)
        logger.info("Kafka: Params %r", params)

        for topics, fn in self.handlers:
            filtered = [t for t in topics if t in only] if only else topics
            for topic in filtered:
                if topic not in self.consumers:
                    logger.info("Kafka: Listen to %s", topic)
                    consumer = self.consumers[topic] = AIOKafkaConsumer(topic, **params)
                    await consumer.start()

                self.map[topic].append(fn)

        self.tasks = [create_task(self.__process__(consumer)) for consumer in self.consumers]

    async def __process__(self, consumer):
        logger = self.app.logger
        logger.info("Start listening Kafka messages")
        async for msg in consumer:
            logger.info("Kafka msg: %s-%s-%s", msg.topic, msg.partition, msg.offset)
            for fn in self.map[msg.topic]:
                try:
                    await fn(msg)
                except Exception as exc:  # noqa: PERF203
                    logger.exception("Kafka: Error while processing message: %r", msg)
                    error_handler = cast(TCallable, self.error_handler)
                    if error_handler:
                        await error_handler(exc)
